import logging

from logilab.common.decorators import monkeypatch
from logilab.common.clcommands import Command
from cubicweb import ConfigurationError
from cubicweb.cwconfig import CubicWebConfiguration

logger = logging.getLogger(__name__)

original_main_run = Command.main_run


@monkeypatch(Command, 'main_run')
def sentry_main_run(self, args, rcfile=None):
    try:
        return original_main_run(self, args, rcfile=rcfile)
    except Exception:
        cmd_args = self.load_command_line_configuration(args)
        if cmd_args:
            try:
                config = CubicWebConfiguration.config_for(cmd_args[0])
            except ConfigurationError:
                pass
            else:
                sentry_dsn = config.get('sentry-dsn')
                if sentry_dsn:
                    from raven import Client
                    client = Client(sentry_dsn)
                    extra = {'args': args}
                    ident = client.get_ident(
                        client.captureException(extra=extra))
                    msg = "Exception caught; reference is %s" % ident
                    logger.critical(msg)
        raise



# copyright 2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.


import logging

import mock

from cubicweb.devtools import testlib

from cubicweb_sentry.patches import (
    add_sentry_support,
    logger as sentry_cube_logger,
)


class LoggerConfigTC(testlib.CubicWebTC):

    @classmethod
    def setUpClass(cls):
        super(LoggerConfigTC, cls).setUpClass()
        add_sentry_support('https://public:secret@example.com/123',
                           logging.ERROR, '1.2.3')

    def test_emit_on_exception(self):
        logger = logging.getLogger('test_logger')
        logger.setLevel(logging.ERROR)
        try:
            raise Exception("Excepted")
        except Exception:
            with mock.patch(
                    'raven.handlers.logging.SentryHandler.emit') as emit:
                logger.exception("Catched")

                emit.assert_called_once()
                self.assertEqual(emit.call_args[0][0].msg, "Catched")

    def test_do_not_emit_on_sentry_cube_logs(self):
        try:
            raise Exception("Excepted")
        except Exception:
            with mock.patch(
                    'raven.handlers.logging.SentryHandler.emit') as emit:
                sentry_cube_logger.exception("Not emitted")

                emit.assert_not_called()



# -*- coding: utf-8 -*-
# copyright 2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-sentry views/forms/actions/components for web ui"""


from cubicweb_sentry.patches import add_sentry_support
import logging


def registration_callback(vreg):
    config = vreg.config
    sentry_dsn = config['sentry-dsn']
    log_level_choice = config['sentry-log-level']
    if log_level_choice:
        log_level = getattr(logging, log_level_choice.upper())
    else:
        log_level = None

    released_cube = config.cubes()[0]
    release = '.'.join(map(str, config.cube_version(released_cube)))

    if sentry_dsn:
        add_sentry_support(sentry_dsn, log_level, release)



# pylint: disable=W0622
"""cubicweb-sentry application packaging information"""

modname = 'sentry'
distname = 'cubicweb-sentry'

numversion = (0, 4, 0)
version = '.'.join(str(num) for num in numversion)

license = 'LGPL'
author = 'LOGILAB S.A. (Paris, FRANCE)'
author_email = 'contact@logilab.fr'
description = 'support for Sentry (getsentry.com)'
web = 'http://www.cubicweb.org/project/%s' % distname

classifiers = [
    'Environment :: Web Environment',
    'Framework :: CubicWeb',
    'Programming Language :: Python',
    'Programming Language :: JavaScript',
]

__depends__ = {'cubicweb': '>= 3.24.0',
               'raven': None}

__recommends__ = {}
